@@ -48,7 +48,7 @@ def check_activation():
     else:
         return create_rest_message('Activation code is required', status_code=StatusCode.BAD_REQUEST)
 
-def get_device_setting(activation_code, device_id):
+def get_device_setting(activation_code, device_id=None):
     if is_not_empty(activation_code):
         db_client = create_db_client(caller_info="get_device_setting")
         with db_client.context():
@@ -93,6 +93,7 @@ def get_device_setting(activation_code, device_id):
         logger.info('activation_code is empty');
         return create_rest_message('Activation code is required', status_code=StatusCode.BAD_REQUEST)
     
+    
 def reset_deviceSetting(activation_code, device_id):
     if is_not_empty(activation_code):
         db_client = create_db_client(caller_info="reset_deviceSetting")
@@ -168,6 +169,43 @@ def updateActivationAndGetDeviceSetting(activation_code, device_id):
     else:
         logger.info('activation_code is empty or device id is empty');
         return create_rest_message('Activation code and device id are required', status_code=StatusCode.BAD_REQUEST)
+
+@loyalty_api_bp.route('/read-settings/activation-code/<activation_code>', methods=['POST'])
+def read_device_setting(activation_code):
+    
+    logger.info('activation_code=%s', activation_code)
+    device_id       = request.args.get('device_id') or request.form.get('device_id') or request.json.get('device_id')
+    
+    if is_not_empty(activation_code):
+        db_client = create_db_client(caller_info="read_device_setting")
+        with db_client.context():
+            device_setting = LoyaltyDeviceSetting.get_by_activation_code(activation_code)
+        
+        if device_setting:
+            logger.info('Found device setting');
+            if device_setting.activated==False:
+            #if True:
+                logger.info('device activation code is valid');
+                device_setting_details = None
+                with db_client.context():
+                    device_setting_details                              = merchant_helpers.construct_setting_by_outlet(device_setting.assigned_outlet_entity, device_setting=device_setting) 
+                    device_setting_details['logo_image_url']            = url_for('system_bp.merchant_logo_image_url', merchant_act_key=device_setting_details.get('account_id'))
+                    device_setting_details['device_id']                 = device_id
+                    
+                logger.debug('device_setting_details=%s', device_setting_details)
+                
+                return create_rest_message(status_code=StatusCode.OK,
+                                               **device_setting_details
+                                               )
+            else:
+                return create_rest_message('The code have been used in other device', status_code=StatusCode.BAD_REQUEST)
+            
+        else:
+            logger.info('Device setting record is not found');
+            return create_rest_message('Invalid activate code', status_code=StatusCode.BAD_REQUEST, error_code=API_ERR_CODE_INVALID_ACTIVATION_CODE)
+    else:
+        logger.info('activation_code is empty');
+        return create_rest_message('Activation code is required', status_code=StatusCode.BAD_REQUEST)
         
     
 @loyalty_api_bp.route('/account-sync', methods=['GET'])
