@@ -18,6 +18,7 @@ from trexmodel.models.datastore.product_models import ProductCatalogue,\
 from trexlib.utils.common.common_util import sort_list
 from firebase_admin import firestore
 from datetime import datetime
+from trexapi.conf import API_ERR_CODE_INVALID_ACTIVATION_CODE
 
 
 #logger = logging.getLogger('api')
@@ -66,7 +67,7 @@ def getPOSAccountSetting(activation_code):
                     #if is_activated:
                     #merchant_acct                                       = pos_setting.merchant_acct_entity
                     outlet                                              = pos_setting.assigned_outlet_entity
-                    pos_setting_details                                 = merchant_helpers.construct_setting_by_outlet(outlet, pos_setting=pos_setting) 
+                    pos_setting_details                                 = merchant_helpers.construct_setting_by_outlet(outlet, device_setting=pos_setting) 
                     
                     pos_setting_details['logo_image_url']               = url_for('system_bp.merchant_logo_image_url', merchant_act_key=pos_setting_details.get('account_id'))
                 
@@ -85,7 +86,7 @@ def getPOSAccountSetting(activation_code):
                 return create_rest_message('The code have been used to activate before', status_code=StatusCode.BAD_REQUEST)
         else:
             logger.info('POS setting record is not found');
-            return create_rest_message('Invalid activate code', status_code=StatusCode.BAD_REQUEST)
+            return create_rest_message('Invalid activation code', status_code=StatusCode.BAD_REQUEST, error_code=API_ERR_CODE_INVALID_ACTIVATION_CODE)
     else:
         logger.info('activation_code is empty');
         return create_rest_message('Activation code is required', status_code=StatusCode.BAD_REQUEST)
@@ -113,7 +114,7 @@ def updateActivationAndGetPOSAccountSetting(activation_code, device_id):
                     pos_setting.activate(device_id)
             
                 if is_valid:    
-                    pos_setting_details                                 = merchant_helpers.construct_setting_by_outlet(pos_setting.assigned_outlet_entity, pos_setting=pos_setting)
+                    pos_setting_details                                 = merchant_helpers.construct_setting_by_outlet(pos_setting.assigned_outlet_entity, device_setting=pos_setting)
                     #pos_setting_details['logo_image_url']               = url_for('system_bp.merchant_logo_image_url', merchant_act_key=pos_setting_details.get('account_id'))
             
             if is_valid:
@@ -274,7 +275,7 @@ def check_catalogue_status():
                         return create_rest_message('Invalid catalogue data', status_code=StatusCode.BAD_REQUEST)
         
         if pos_setting is None:
-            return create_rest_message('Invalid activation code', status_code=StatusCode.BAD_REQUEST)
+            return create_rest_message('Invalid activation code', status_code=StatusCode.BAD_REQUEST, err_code=API_ERR_CODE_INVALID_ACTIVATION_CODE)
         else:
             return create_rest_message('Not catqalogue have been assigned or published', status_code=StatusCode.BAD_REQUEST)
                 
@@ -334,7 +335,7 @@ def get_catalogue():
             else:
                 if pos_setting is None:
                     logger.debug('Invalid activation code');
-                    return create_rest_message('Invalid activation code', status_code=StatusCode.BAD_REQUEST)
+                    return create_rest_message('Invalid activation code', status_code=StatusCode.BAD_REQUEST, err_code=API_ERR_CODE_INVALID_ACTIVATION_CODE)
                 else:
                     logger.debug('No catalogue have been assigned or published');
                     return create_rest_message('No catqalogue have been assigned or published', status_code=StatusCode.BAD_REQUEST)
@@ -367,7 +368,7 @@ def version_sync():
                                         ]
                                         
                                 '''
-                                'cart'  :[
+                                'setting'  :[
                                             
                                         ]
                                        
