@@ -17,7 +17,8 @@ from trexmodel.models.datastore.customer_model_helpers import update_customer_en
     update_reward_summary_with_new_reward,\
     update_prepaid_summary_with_new_prepaid,\
     update_customer_entiteld_voucher_summary_with_customer_new_voucher,\
-    update_prepaid_summary_with_reverted_prepaid
+    update_prepaid_summary_with_reverted_prepaid,\
+    update_reward_summary_with_reverted_reward
 from trexanalytics.bigquery_upstream_data_config import create_merchant_customer_reward_reverted_upstream_for_merchant,\
     create_merchant_customer_transaction_upstream_for_merchant,\
     create_merchant_customer_redemption_upstream_for_merchant,\
@@ -29,8 +30,11 @@ from trexmodel.models.datastore.transaction_models import CustomerTransaction
 from trexmodel.models.datastore.model_decorators import model_transactional
 from trexmodel.models.datastore.prepaid_models import CustomerPrepaidReward
 from trexadmin.libs.app.reward_program.reward_program_base import EntitledVoucherSummary
-from trexmodel.models.datastore.customer_models import Customer
+from trexmodel.models.datastore.customer_models import Customer,\
+    CustomerMembership
 from trexmodel.models.datastore.redeem_models import CustomerRedemption
+from trexlib.utils.string_util import is_not_empty
+from trexlib.utils.log_util import get_tracelog
 
 logger = logging.getLogger('debug')
 #logger = logging.getLogger('debug')
@@ -74,7 +78,7 @@ def create_sales_transaction(customer, transact_outlet=None, sales_amount=.0, ta
         
     return __start_transaction_for_customer_transaction()
 
-def create_non_sales_system_transaction(customer, transact_datetime=None, remarks=None):
+def create_non_sales_system_transaction(customer, transact_datetime=None, remarks=None, system_remarks=None):
     
     logger.debug('---create_sales_transaction---')
     
@@ -86,6 +90,7 @@ def create_non_sales_system_transaction(customer, transact_datetime=None, remark
                                        allow_to_revert      = False,
                                        is_sales_transaction = False,
                                        remarks              = remarks,
+                                       system_remarks       = system_remarks,
                                        )
             
         return customer_transaction
@@ -449,10 +454,11 @@ def check_spending_reward_for_transaction(customer_acct, transaction_details, pr
     return give_reward_status
     
 def check_giveaway_reward_for_transaction(customer_acct, transaction_details, program_configuration_list=None, reward_set=1):
-    merchant_acct   = transaction_details.transact_merchant_acct
-    currency_code   = merchant_acct.currency_code
+    merchant_acct                   = transaction_details.transact_merchant_acct
+    program_configuration_list      = merchant_acct.program_configuration_list if program_configuration_list is None else program_configuration_list
+    currency_code                   = merchant_acct.currency_code
     
-    currency_config = get_currency_config(currency_code)
+    currency_config                 = get_currency_config(currency_code)
     
     give_reward_status =  RewardProgramFactory().get_giveaway_reward(customer_acct, 
                                                             transaction_details, 
@@ -466,6 +472,27 @@ def check_giveaway_reward_for_transaction(customer_acct, transaction_details, pr
     
     return give_reward_status
 
+def check_giveaway_reward_for_membership_purchase_transaction(customer_acct, transaction_details, program_configuration_list=None, reward_set=1):
+    logger.debug('---check_giveaway_reward_for_membership_purchase_transaction---')
+    merchant_acct                   = transaction_details.transact_merchant_acct
+    program_configuration_list      = merchant_acct.program_configuration_list if program_configuration_list is None else program_configuration_list
+    
+    currency_code   = merchant_acct.currency_code
+    
+    currency_config = get_currency_config(currency_code)
+    
+    give_reward_status =  RewardProgramFactory().get_purchase_membership_giveaway_reward(customer_acct, 
+                                                            transaction_details, 
+                                                            program_configuration_list=program_configuration_list, 
+                                                            currency=currency_config, 
+                                                            reward_set=reward_set)
+    
+    '''
+    if give_reward_status:
+        #check if the giveaway reward format match tier reward program setting
+        check_tier_reward_for_transaction(customer_acct, transaction_details)
+    '''
+    return give_reward_status
 
 def redeem_reward_transaction(customer, redeem_outlet=None, reward_format=None, reward_amount=.0, invoice_id=None, remarks=None, 
                             redeemed_by=None, redeemed_datetime=None, transaction_id=None,
@@ -531,24 +558,37 @@ def update_customer_prepaid_summary(customer):
     
     customer.prepaid_summary = prepaid_summary
     customer.put()
+    
+def update_customer_memberships_list(customer):
+    customer_memberships_list     = CustomerMembership.list_by_customer(customer)
+    memberships_list              = []
+    
+    
+    for customer_membership in  customer_memberships_list:
+        if customer_membership.is_valid():
+            memberships_list.append(customer_membership.merchant_membership_key)
+        
+        
+    customer.memberships_list    = memberships_list
+    
+    customer.put()    
 
 def revert_transaction(transaction_details, reverted_by, reverted_datetime=None):
     merchant_acct                       = transaction_details.transact_merchant_acct
     transaction_id                      = transaction_details.transaction_id
     customer_acct                       = transaction_details.transact_customer_acct
-    is_stamp_entitled_in_transaction    = False
-    is_point_entitled_in_transaction    = False
-    is_voucher_entitled_in_transaction  = False
-    is_prepaid_entitled_in_transaction  = False
         
-    if transaction_details.entitled_reward_summary is not None:
-        is_stamp_entitled_in_transaction    = transaction_details.entitled_reward_summary.get(program_conf.REWARD_FORMAT_STAMP) is not None
-        is_point_entitled_in_transaction    = transaction_details.entitled_reward_summary.get(program_conf.REWARD_FORMAT_POINT) is not None
-        is_voucher_entitled_in_transaction  = transaction_details.entitled_voucher_summary is not None
+        
+        
+    
+    is_stamp_entitled_in_transaction    = transaction_details.entitled_reward_summary.get(program_conf.REWARD_FORMAT_STAMP) is not None
+    is_point_entitled_in_transaction    = transaction_details.entitled_reward_summary.get(program_conf.REWARD_FORMAT_POINT) is not None
+    is_voucher_entitled_in_transaction  = transaction_details.entitled_voucher_summary is not None
+    is_prepaid_entitled_in_transaction  = transaction_details.entitled_prepaid_summary is not None
     
     
-    if transaction_details.entitled_prepaid_summary is not None:
-        is_prepaid_entitled_in_transaction  = True
+    logger.debug('transaction_details.entitled_reward_summary=%s', transaction_details.entitled_reward_summary)
+    logger.debug('transaction_details.entitled_prepaid_summary=%s', transaction_details.entitled_prepaid_summary)
     
     
     reverting_point_details_list        = CustomerPointReward.list_by_transaction_id(transaction_id) if is_point_entitled_in_transaction else []
@@ -557,6 +597,7 @@ def revert_transaction(transaction_details, reverted_by, reverted_datetime=None)
     reverting_prepaid_list              = CustomerPrepaidReward.list_by_transaction_id(transaction_id) if is_prepaid_entitled_in_transaction else []
     
     is_transaction_reward_used          = False
+    customer_reward_summary             = customer_acct.reward_summary or {}
     customer_prepaid_summary            = customer_acct.prepaid_summary or {}
     entitled_voucher_summary            = customer_acct.entitled_voucher_summary or {}
     
@@ -615,6 +656,8 @@ def revert_transaction(transaction_details, reverted_by, reverted_datetime=None)
                 p.reverted_by_username          = reverted_by.username
                 p.put()
                 
+                customer_reward_summary = update_reward_summary_with_reverted_reward(customer_reward_summary, p.to_reward_summary())
+                
                 create_merchant_customer_reward_reverted_upstream_for_merchant(transaction_details, p, reverted_datetime)
                 
             
@@ -626,6 +669,8 @@ def revert_transaction(transaction_details, reverted_by, reverted_datetime=None)
                 p.reverted_by_username          = reverted_by.username
                 p.put()
                 
+                customer_reward_summary = update_reward_summary_with_reverted_reward(customer_reward_summary, p.to_reward_summary())
+                
                 create_merchant_customer_reward_reverted_upstream_for_merchant(transaction_details, p, reverted_datetime)
             
         for p in reverting_vouchers_list:
@@ -659,7 +704,8 @@ def revert_transaction(transaction_details, reverted_by, reverted_datetime=None)
         
         
         if is_stamp_entitled_in_transaction or is_point_entitled_in_transaction:
-            update_customer_reward_summary(customer_acct)
+            #update_customer_reward_summary(customer_acct)
+            customer_acct.reward_summary           = customer_reward_summary
         
         if is_prepaid_entitled_in_transaction:
             #update_customer_prepaid_summary(customer_acct)
@@ -690,14 +736,21 @@ def revert_transaction(transaction_details, reverted_by, reverted_datetime=None)
         transaction_topup_prepaid       = .0
         transaction_entitled_prepaid    = .0
         
+        logger.debug('is_point_entitled_in_transaction=%s', is_point_entitled_in_transaction)
+        logger.debug('is_stamp_entitled_in_transaction=%s', is_stamp_entitled_in_transaction)
+        logger.debug('is_prepaid_entitled_in_transaction=%s', is_prepaid_entitled_in_transaction)
+        
         if is_point_entitled_in_transaction:
-            transaction_entitled_point = transaction_details.entitled_reward_summary.get(program_conf.REWARD_FORMAT_POINT).get('amount')
+            logger.debug('transaction_details entitled point summary=%s', transaction_details.entitled_reward_summary.get(program_conf.REWARD_FORMAT_POINT))
+            transaction_entitled_point = transaction_details.entitled_reward_summary.get(program_conf.REWARD_FORMAT_POINT).get('amount') or 0
             
         if is_stamp_entitled_in_transaction:
-            transaction_entitled_stamp = transaction_details.entitled_reward_summary.get(program_conf.REWARD_FORMAT_STAMP).get('amount')
+            logger.debug('transaction_details entitled stamp summary=%s', transaction_details.entitled_reward_summary.get(program_conf.REWARD_FORMAT_STAMP))
+            transaction_entitled_stamp = transaction_details.entitled_reward_summary.get(program_conf.REWARD_FORMAT_STAMP).get('amount') or 0
             
         if is_prepaid_entitled_in_transaction:
-            transaction_topup_prepaid = transaction_details.entitled_prepaid_summary.get('amount')
+            logger.debug('transaction_details entitled prepaid summary=%s', transaction_details.entitled_prepaid_summary)
+            transaction_topup_prepaid = transaction_details.entitled_prepaid_summary.get('amount') or .0
             
         customer_acct.last_transact_datetime    = customer_acct.previous_transact_datetime
         total_transact_amount                   -= transaction_details.transact_amount
@@ -727,6 +780,21 @@ def revert_transaction(transaction_details, reverted_by, reverted_datetime=None)
         customer_acct.kpi_summary['total_accumulated_topup']        = total_accumulated_topup
         customer_acct.kpi_summary['total_accumulated_prepaid']      = total_accumulated_prepaid
         
+        
+        
+        if transaction_details.is_membership_purchase:
+            #remove membership
+            purchased_customer_membership   = transaction_details.purchased_customer_membership_entity
+            merchant_membership_key         = purchased_customer_membership.merchant_membership_key 
+            purchased_customer_membership.delete()
+            
+            customer_acct.memberships_list.remove(merchant_membership_key)
+            
+        elif transaction_details.is_membership_renew:
+            #revert renewal
+            renewed_customer_membership = transaction_details.purchased_customer_membership_entity
+            renewed_customer_membership.revert_renewal()
+        
         customer_acct.put()
          
         check_for_tier_membership_upgrade_downgrade(customer_acct, merchant_acct, transaction_details) 
@@ -930,7 +998,10 @@ def giveaway_birthday_reward_to_customer(customer, program_configuration, transa
         logger.debug('------> Customer(%s) entitled before (%s) =%s %s', customer.name, program_desc)
         
         if is_entitled_before == False:
-            transaction_details = create_non_sales_system_transaction(customer, transact_datetime=transact_datetime, remarks=remarks)   
+            transaction_details = create_non_sales_system_transaction(customer, transact_datetime=transact_datetime, remarks=remarks)
+            
+            logger.debug('transaction_details=%s', transaction_details)
+               
             currency_config = get_currency_config(currency_code)
             RewardProgramFactory().get_giveaway_based_reward(customer, transaction_details, 
                                                                                 program_configuration_list  = [program_configuration], 
@@ -942,6 +1013,40 @@ def giveaway_birthday_reward_to_customer(customer, program_configuration, transa
     except:
         logger.error('Failed to process for customer=%s', customer.name)
         
+@model_transactional(desc="giveaway_membership_reward_to_customer")                
+def giveaway_membership_reward_to_customer(customer, program_configuration, transact_datetime, merchant_acct ):
+    
+    try:
+        this_year               = transact_datetime.year
+        program_key             = program_configuration.get('program_key')
+        program_label           = program_configuration.get('label')
+        remarks                 = program_configuration.get('remarks')
+        program_desc            = program_configuration.get('desc')
+        currency_code           = merchant_acct.currency_code
+        
+        logger.debug('------> going to check for program_key=%s for %s', program_key, customer.name)
+        
+        is_entitled_before = Customer.check_membership_year_reward_have_entitled_before(customer, this_year, program_key)
+        logger.debug('------> Customer(%s) entitled %s', customer.name, program_desc)
+        
+        if is_entitled_before == False:
+            system_remarks = 'Giveaway reward from %s' % program_label
+            transaction_details = create_non_sales_system_transaction(customer, remarks=remarks, system_remarks=system_remarks)
+            
+            logger.debug('transaction_details=%s', transaction_details)
+               
+            currency_config = get_currency_config(currency_code)
+            RewardProgramFactory().get_giveaway_reward(customer, transaction_details, 
+                                                                                program_configuration_list  = [program_configuration], 
+                                                                                currency                    = currency_config
+                                                                                )
+            Customer.update_customer_entitled_membership_reward_summary(customer, program_key)
+        else:
+            logger.info('Customer(%s) have entitled membership reward (%s) on year %d', customer.name, program_desc, this_year)
+    except:
+        logger.error('Failed to process for customer=%s, due to %s', customer.name, get_tracelog())  
+        raise      
+        
 
 def revert_redemption(redemption_details, reverted_by, reverted_datetime=None):
     
